from fastapi import APIRouter, UploadFile, File, HTTPException
import logging
from app.services.spreadsheet_service import SampleSpreadsheetImporter, SpreadsheetImportError
from fastapi.responses import FileResponse
import os

router = APIRouter()
logger = logging.getLogger(__name__)


@router.get("/download-template", response_class=FileResponse)
async def download_template():
    # Constructing the absolute path
    current_dir = os.path.dirname(__file__)
    template_path = os.path.join(current_dir, "../../downloads/V7_TELLSamplesSpreadsheetTemplate.xlsx")

    if not os.path.exists(template_path):
        raise HTTPException(status_code=404, detail="Template file not found.")

    return FileResponse(template_path, filename="V7_TELLSamplesSpreadsheetTemplate.xlsx",
                        media_type="application/vnd.openxmlformats-officedocument.spreadsheetml.sheet")


@router.post("/upload")
async def upload_file(file: UploadFile = File(...)):
    try:
        logger.info(f"Received file: {file.filename}")

        # File type check
        if not file.filename.endswith('.xlsx'):
            logger.error("Invalid file format")
            raise HTTPException(status_code=400, detail="Invalid file format. Please upload an .xlsx file.")

        # Reading file
        importer = SampleSpreadsheetImporter()
        validated_model = importer.import_spreadsheet(file)
        logger.info(f"Validated model: {validated_model}")

        dewars = {sample.dewarname for sample in validated_model if sample.dewarname}
        pucks = {sample.puckname for sample in validated_model if sample.puckname}
        samples = {sample.crystalname for sample in validated_model if sample.crystalname}

        # Logging the sets of names
        logger.info(f"Dewar Names: {dewars}")
        logger.info(f"Puck Names: {pucks}")
        logger.info(f"Sample Names: {samples}")

        # Forming structured response
        response = {
            "dewars_count": len(dewars),
            "dewars": list(dewars),
            "pucks_count": len(pucks),
            "pucks": list(pucks),
            "samples_count": len(samples),
            "samples": list(samples)
        }

        logger.info(f"Returning response: {response}")
        return response
    except SpreadsheetImportError as e:
        logger.error(f"Spreadsheet import error: {str(e)}")
        raise HTTPException(status_code=400, detail=str(e))
    except Exception as e:
        logger.error(f"Failed to process file: {str(e)}")
        raise HTTPException(status_code=500, detail=f"Failed to upload file. Please try again. {str(e)}")
